package sw.programme.cordova.plugin;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.util.Log;

import com.cipherlab.barcode.GeneralString;
import com.cipherlab.barcode.ReaderManager;
import com.cipherlab.barcode.decoder.KeyboardEmulationType;
import com.cipherlab.barcode.decoderparams.ReaderOutputConfiguration;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaInterface;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.PluginResult;
import org.json.JSONArray;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

public class CipherLabBarcodeReader extends CordovaPlugin {

    private static final String TAG = "CipherLabBarcodeReader";
    private static final String ACTION_SET_BARCODE_SCANNED_LISTENER = "setBarcodeScannedListener";
    private static final String ACTION_CLEAR_BARCODE_SCANNED_LISTENER = "clearBarcodeScannedListener";

    private ReaderManager readerManager;
    private CipherLabDataReceiver dataReceiver;

    @Override
    public boolean execute(String action, JSONArray args, CallbackContext callbackContext) {
        if (action.equals(ACTION_SET_BARCODE_SCANNED_LISTENER)) {
            if (readerManager == null) {
              initReaderManager();
            }

            if (dataReceiver == null) {
              initDataReceiver();
            }

            dataReceiver.setCallbackContext(callbackContext);
            return true;

        } else if (action.equals(ACTION_CLEAR_BARCODE_SCANNED_LISTENER)) {
            if (dataReceiver == null) {
              sendErrorResponse(callbackContext, "Barcode scanned event listener was not set");
              return true;
            }

            dataReceiver.setCallbackContext(null);
            return true;

        }

        return false;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        cordova.getActivity().unregisterReceiver(dataReceiver);
    }

    private void initReaderManager() {
        if (readerManager != null) {
            return;
        }

        readerManager = ReaderManager.InitInstance(cordova.getActivity());
    }

    private void initDataReceiver() {
        if (dataReceiver != null) {
            return;
        }

        IntentFilter filter = new IntentFilter();
        filter.addAction(GeneralString.Intent_READERSERVICE_CONNECTED);
        filter.addAction(GeneralString.Intent_PASS_TO_APP);

        dataReceiver = new CipherLabDataReceiver(cordova);
        cordova.getActivity().registerReceiver(dataReceiver, filter);
    }

    private void sendErrorResponse(final CallbackContext callbackContext, final String message) {
        Log.e(TAG, message);
        cordova.getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                callbackContext.error(message);
            }
        });
    }

    private class CipherLabDataReceiver extends BroadcastReceiver {

        private final CordovaInterface cordova;
        private CallbackContext callbackContext;

        CipherLabDataReceiver(CordovaInterface cordova) {
            this.cordova = cordova;
        }

        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getAction().equals(GeneralString.Intent_READERSERVICE_CONNECTED)) {

                ReaderOutputConfiguration settings = new ReaderOutputConfiguration();
                readerManager.Get_ReaderOutputConfiguration(settings);
                settings.enableKeyboardEmulation = KeyboardEmulationType.None;
                readerManager.Set_ReaderOutputConfiguration(settings);

            } else if (intent.getAction().equals(GeneralString.Intent_PASS_TO_APP)) {

                String barcode = intent.getStringExtra(GeneralString.BcReaderData);
                String symbology = intent.getStringExtra(GeneralString.BcReaderCodeTypeStr);
                int codeType = intent.getIntExtra(GeneralString.BcReaderCodeType, 0);

                Log.d(TAG, "Barcode: " + barcode + ", Symbology: " + symbology);

                if (callbackContext == null) {
                    Log.d(TAG, "Barcode scanned event listener is not set");
                    return;
                }

                try {

                    JSONObject data = new JSONObject();
                    data.put("barcode", barcode);
                    data.put("symbology", symbology);
                    data.put("codeType", codeType);

                    Log.d(TAG, "data: " + data.toString());

                    final PluginResult pluginResult = new PluginResult(PluginResult.Status.OK, data);
                    pluginResult.setKeepCallback(true);

                    cordova.getActivity().runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            callbackContext.sendPluginResult(pluginResult);
                        }
                    });

                } catch (Exception e) {
                    Log.e(TAG, e.toString());
                    final PluginResult pluginResult = new PluginResult(PluginResult.Status.ERROR, "Barcode decoding error");
                    pluginResult.setKeepCallback(true);
                    cordova.getActivity().runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                          callbackContext.sendPluginResult(pluginResult);
                        }
                    });
                }

            }
        }

        public void setCallbackContext(CallbackContext callbackContext) {
            this.callbackContext = callbackContext;
        }

    }
}
